(define-module (ordo inventory)
  #:use-module (ice-9 match)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-9)
  #:use-module (srfi srfi-69)
  #:use-module (logging logger)
  #:use-module (ordo connection)
  #:export (make-host
            host?
            host-name
            host-connection
            host-tags
            add-host!
            resolve-hosts))

(define *hosts* (make-hash-table equal?))

(define-record-type <host>
  (make-host name connection tags)
  host?
  (name host-name)
  (connection host-connection)
  (tags host-tags))

(define (add-host! name connection . tags)
  (log-msg 'DEBUG "Adding host to inventory: " name)
  (hash-table-set! *hosts* name (make-host name connection tags)))

(define (tagged-all? wanted-tags)
  (lambda (h)
    (lset= equal? wanted-tags (lset-intersection equal? (host-tags h) wanted-tags))))

(define (tagged-any? wanted-tags)
  (lambda (h)
    (not (null? (lset-intersection equal? (host-tags h) wanted-tags)))))

(define resolve-hosts
  (match-lambda
    ("localhost" (list (or (hash-table-ref/default *hosts* "localhost" #f)
                           (make-host "localhost" (local-connection) '()))))
    ((? string? name) (list (hash-table-ref *hosts* name)))
    ('all (hash-table-values *hosts*))
    (('every-tag tag . tags) (filter (tagged-all? (cons tag tags)) (hash-table-values *hosts*)))
    (('any-tag tag . tags) (filter (tagged-any? (cons tag tags)) (hash-table-values *hosts*)))))

#!
(define (setup-test-data)
  (add-host! "little-rascal" (ssh-connection "ray" "little-rascal") #:linux #:guix)
  (add-host! "linux-1" (ssh-connection "root" "linux-1") #:linux)
  (add-host! "linux-2" (ssh-connection "root" "linux-2") #:linux)
  (add-host! "debian-1" (ssh-connection "root" "debian-1") #:linux #:debian)
  (add-host! "debian-2" (ssh-connection "root" "debian-2") #:linux #:debian)
  (add-host! "debian-3" (ssh-connection "root" "debian-3") #:linux #:debian #:eu-west-1))
!#
